@@ -19,14 +19,14 @@ from Py6S import *
 import numpy as np
 from matplotlib.pyplot import *
 import copy
-
+import sys
 
 class Wavelengths:
 
     """Helper functions for running the 6S model for a range of wavelengths, and plotting the result"""
 
     @classmethod
-    def run_wavelengths(cls, s, wavelengths, output_name=None, n=None):
+    def run_wavelengths(cls, s, wavelengths, output_name=None, n=None, verbose=False):
         """Runs the given SixS parameterisation for each of the wavelengths given, optionally extracting a specific output.
 
         This function is used by all of the other wavelengths running functions, such as :method:`run_vnir`, and thus
@@ -41,6 +41,7 @@ class Wavelengths:
         * ``wavelengths`` -- An iterable containing the wavelengths to iterate over
         * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
         * ``n`` -- (Optional) The number of threads to run in parallel. This defaults to the number of CPU cores in your system, and is unlikely to need changing.
+        * ``verbose`` -- (Optional) Print wavelengths as Py6S is running (default=False)
 
         Return value:
 
@@ -62,12 +63,13 @@ class Wavelengths:
             s.outputs = None
             a = copy.deepcopy(s)
             a.wavelength = Wavelength(wv)
-            print wv
+            if verbose:
+               print(wv)
             a.run()
             if output_name is None:
                 return a.outputs
             else:
-                return getattr(a.outputs, output_name)
+                return cls.recursive_getattr(a.outputs, output_name)
 
         # Run the map
         from multiprocessing.dummy import Pool
@@ -77,12 +79,20 @@ class Wavelengths:
         else:
             pool = Pool(n)
 
-        print "Running for many wavelengths - this may take a long time"
+        if verbose:
+            print("wavelengths pass:")
+            print(wavelengths)
+            print(type(wavelengths))
+
+        print("Running for many wavelengths - this may take a long time")
         results = pool.map(f, wavelengths)
 
+        pool.close()
+        pool.join()
+
         try:
             if len(wavelengths[0]) == 4:
-                cleaned_wavelengths = map(lambda x: x[:3], wavelengths)
+                cleaned_wavelengths = list(map(lambda x: x[:3], wavelengths))
                 return np.array(cleaned_wavelengths), np.array(results)
             else:
                 return np.array(wavelengths), np.array(results)
@@ -178,6 +188,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -201,12 +214,17 @@ class Wavelengths:
               PredefinedWavelengths.LANDSAT_OLI_B4,
               PredefinedWavelengths.LANDSAT_OLI_B5,
               PredefinedWavelengths.LANDSAT_OLI_B6,
-              PredefinedWavelengths.LANDSAT_OLI_B7]
+              PredefinedWavelengths.LANDSAT_OLI_B7,
+              PredefinedWavelengths.LANDSAT_OLI_B8,
+              PredefinedWavelengths.LANDSAT_OLI_B9]
 
         wv, res = cls.run_wavelengths(s, wv, **kwargs)
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -228,6 +246,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -249,8 +270,239 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
+    @classmethod
+    def run_s2a_msi(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel2/MSI bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S2A_MSI_01,
+              PredefinedWavelengths.S2A_MSI_02,
+              PredefinedWavelengths.S2A_MSI_03,
+              PredefinedWavelengths.S2A_MSI_04,
+              PredefinedWavelengths.S2A_MSI_05,
+              PredefinedWavelengths.S2A_MSI_06,
+              PredefinedWavelengths.S2A_MSI_07,
+              PredefinedWavelengths.S2A_MSI_08,
+              PredefinedWavelengths.S2A_MSI_8A,
+              PredefinedWavelengths.S2A_MSI_09,
+              PredefinedWavelengths.S2A_MSI_10,
+              PredefinedWavelengths.S2A_MSI_11,
+              PredefinedWavelengths.S2A_MSI_12]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+    @classmethod
+    def run_s2b_msi(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel2/MSI bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S2B_MSI_01,
+              PredefinedWavelengths.S2B_MSI_02,
+              PredefinedWavelengths.S2B_MSI_03,
+              PredefinedWavelengths.S2B_MSI_04,
+              PredefinedWavelengths.S2B_MSI_05,
+              PredefinedWavelengths.S2B_MSI_06,
+              PredefinedWavelengths.S2B_MSI_07,
+              PredefinedWavelengths.S2B_MSI_08,
+              PredefinedWavelengths.S2B_MSI_8A,
+              PredefinedWavelengths.S2B_MSI_09,
+              PredefinedWavelengths.S2B_MSI_10,
+              PredefinedWavelengths.S2B_MSI_11,
+              PredefinedWavelengths.S2B_MSI_12]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
+    @classmethod
+    def run_s3a_olci(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel3/OLCI bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S3A_OLCI_01,
+              PredefinedWavelengths.S3A_OLCI_02,
+              PredefinedWavelengths.S3A_OLCI_03,
+              PredefinedWavelengths.S3A_OLCI_04,
+              PredefinedWavelengths.S3A_OLCI_05,
+              PredefinedWavelengths.S3A_OLCI_06,
+              PredefinedWavelengths.S3A_OLCI_07,
+              PredefinedWavelengths.S3A_OLCI_08,
+              PredefinedWavelengths.S3A_OLCI_09,
+              PredefinedWavelengths.S3A_OLCI_10,
+              PredefinedWavelengths.S3A_OLCI_11,
+              PredefinedWavelengths.S3A_OLCI_12,
+              PredefinedWavelengths.S3A_OLCI_13,
+              PredefinedWavelengths.S3A_OLCI_14,
+              PredefinedWavelengths.S3A_OLCI_15,
+              PredefinedWavelengths.S3A_OLCI_16,
+              PredefinedWavelengths.S3A_OLCI_17,
+              PredefinedWavelengths.S3A_OLCI_18,
+              PredefinedWavelengths.S3A_OLCI_19,
+              PredefinedWavelengths.S3A_OLCI_20,
+              PredefinedWavelengths.S3A_OLCI_21]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
+    @classmethod
+    def run_s3a_slstr(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel3/SLSTR bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S3A_SLSTR_01,
+              PredefinedWavelengths.S3A_SLSTR_02,
+              PredefinedWavelengths.S3A_SLSTR_03,
+              PredefinedWavelengths.S3A_SLSTR_04,
+              PredefinedWavelengths.S3A_SLSTR_05,
+              PredefinedWavelengths.S3A_SLSTR_06]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+    @classmethod
+    def run_s3b_olci(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel3/OLCI bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S3B_OLCI_01,
+              PredefinedWavelengths.S3B_OLCI_02,
+              PredefinedWavelengths.S3B_OLCI_03,
+              PredefinedWavelengths.S3B_OLCI_04,
+              PredefinedWavelengths.S3B_OLCI_05,
+              PredefinedWavelengths.S3B_OLCI_06,
+              PredefinedWavelengths.S3B_OLCI_07,
+              PredefinedWavelengths.S3B_OLCI_08,
+              PredefinedWavelengths.S3B_OLCI_09,
+              PredefinedWavelengths.S3B_OLCI_10,
+              PredefinedWavelengths.S3B_OLCI_11,
+              PredefinedWavelengths.S3B_OLCI_12,
+              PredefinedWavelengths.S3B_OLCI_13,
+              PredefinedWavelengths.S3B_OLCI_14,
+              PredefinedWavelengths.S3B_OLCI_15,
+              PredefinedWavelengths.S3B_OLCI_16,
+              PredefinedWavelengths.S3B_OLCI_17,
+              PredefinedWavelengths.S3B_OLCI_18,
+              PredefinedWavelengths.S3B_OLCI_19,
+              PredefinedWavelengths.S3B_OLCI_20,
+              PredefinedWavelengths.S3B_OLCI_21]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
+    @classmethod
+    def run_s3b_slstr(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Sentinel3/SLSTR bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.S3B_SLSTR_01,
+              PredefinedWavelengths.S3B_SLSTR_02,
+              PredefinedWavelengths.S3B_SLSTR_03,
+              PredefinedWavelengths.S3B_SLSTR_04,
+              PredefinedWavelengths.S3B_SLSTR_05,
+              PredefinedWavelengths.S3B_SLSTR_06]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
     @classmethod
     def run_meris(cls, s, **kwargs):
         """Runs the given SixS parameterisation for all of the MERIS bands within the 6S band range, optionally extracting a specific output.
@@ -270,6 +522,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -291,8 +546,85 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
+    @classmethod
+    def run_aqua(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the MODIS bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.ACCURATE_MODIS_AQUA_1,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_2,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_3,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_4,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_5,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_6,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_7,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_11,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_12,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_13,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_14,
+              PredefinedWavelengths.ACCURATE_MODIS_AQUA_15]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
+    @classmethod
+    def run_terra(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the MODIS bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.ACCURATE_MODIS_TERRA_1,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_2,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_3,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_4,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_5,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_6,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_7,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_11,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_12,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_13,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_14,
+              PredefinedWavelengths.ACCURATE_MODIS_TERRA_15]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+
     @classmethod
     def run_spot_hrv(cls, s, **kwargs):
         """Runs the given SixS parameterisation for all of the SPOT HRV (both 1 and 2, as the only bands specified are the same for both) bands within the 6S band range, optionally extracting a specific output.
@@ -312,6 +644,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -333,6 +668,93 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+    @classmethod
+    def run_probav_1(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Proba-V left camera bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.PROBAV_1_01,
+              PredefinedWavelengths.PROBAV_1_02,
+              PredefinedWavelengths.PROBAV_1_03,
+              PredefinedWavelengths.PROBAV_1_04]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+    @classmethod
+    def run_probav_2(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Proba-V middle camera bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.PROBAV_2_01,
+              PredefinedWavelengths.PROBAV_2_02,
+              PredefinedWavelengths.PROBAV_2_03,
+              PredefinedWavelengths.PROBAV_2_04]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
+        return (centre_wvs, res)
+
+    @classmethod
+    def run_probav_3(cls, s, **kwargs):
+        """Runs the given SixS parameterisation for all of the Proba-V right camera bands within the 6S band range, optionally extracting a specific output.
+
+        Arguments:
+
+        * ``s`` -- A :class:`.SixS` instance with the parameters set as required
+        * ``output_name`` -- (Optional) The output to extract from ``s.outputs``, as a string that could be placed after ``s.outputs.``, for example ``pixel_reflectance``
+
+        Return value:
+
+        A tuple containing the centre wavlengths used for the run and the results of the simulations. The results will be a list of :class:`SixS.Outputs` instances if ``output_name`` is not set,
+        or a list of values of the selected output if ``output_name`` is set.
+        """
+        wv = [PredefinedWavelengths.PROBAV_3_01,
+              PredefinedWavelengths.PROBAV_3_02,
+              PredefinedWavelengths.PROBAV_3_03,
+              PredefinedWavelengths.PROBAV_3_04]
+
+        wv, res = cls.run_wavelengths(s, wv, **kwargs)
+
+        centre_wvs = map(cls.to_centre_wavelengths, wv)
+
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -354,6 +776,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -375,6 +800,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -396,6 +824,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -417,6 +848,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -438,6 +872,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -459,6 +896,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -480,6 +920,9 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
     @classmethod
@@ -501,8 +944,20 @@ class Wavelengths:
 
         centre_wvs = map(cls.to_centre_wavelengths, wv)
 
+        if sys.version_info[0] >= 3:
+            centre_wvs = list(centre_wvs)
+
         return (centre_wvs, res)
 
+    @classmethod
+    def recursive_getattr(cls, obj, attr):
+        prev_part = obj
+
+        for part in attr.split("."):
+            prev_part = getattr(prev_part, part)
+
+        return prev_part
+
     @classmethod
     def extract_output(cls, results, output_name):
         """Extracts data for one particular SixS output from a list of SixS.Outputs instances.
@@ -515,7 +970,7 @@ class Wavelengths:
         * ``output_name`` -- The name of the output to extract. This should be a string containing whatever is put after the `s.outputs` when printing the output, for example `'pixel_reflectance'`.
 
         """
-        results_output = [getattr(r, output_name) for r in results]
+        results_output = [cls.recursive_getattr(r, output_name) for r in results]
 
         return results_output
 
