@@ -16,12 +16,17 @@
 # along with Py6S.  If not, see <http://www.gnu.org/licenses/>.
 
 from Py6S import *
+import sys
 import numpy as np
 import urllib
 import re
-import StringIO
 from scipy.interpolate import interp1d
+import io
 
+if sys.version_info[0] >= 3:
+    import urllib.request as urllib
+else:
+    import urllib
 
 class Radiosonde:
 
@@ -297,6 +302,8 @@ class Radiosonde:
             raise ParameterException("url", "The URL for importing radiosonde data is not giving a valid response")
 
         html = u.read()
+        if sys.version_info[0] >= 3:
+            html = html.decode()
 
         if "Sorry, the server is too busy to process your request" in html:
             raise ParameterException("url", "The server is too busy")
@@ -311,9 +318,15 @@ class Radiosonde:
         spl = spl[:-1]
         table = "\n".join(spl)
 
+        # Check for partly empty first line in U of W data which impacts interpolations:
+        if len(table.split('\n')[4].split()) != 11:
+            num_skip = 5
+        else:
+            num_skip = 4
+
         # Import to NumPy arrays
-        s = StringIO.StringIO(table)
-        array = np.loadtxt(s, skiprows=4, usecols=(0, 1, 2, 5))
+        s = io.BytesIO(table.encode())
+        array = np.genfromtxt(s,skip_header=num_skip, delimiter=7,usecols=(0, 1, 2, 5),filling_values=0)
 
         pressure = array[:, 0]
         altitude = array[:, 1] / 1000
